#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include "config.h"
#include "types.h"
#include "adc.h"
#include "lcd.h"
#include "timer0.h"
#include "encoder.h"
#include "spi.h"

//#define USELCD

void updateEncoder(uint8_t r, uint16_t val) {
	if (val!=0) {
		reg[r] += val;
		regUpdate[r] = 1;
	}
}

void updateADC(uint8_t r, uint8_t adc) {
	uint16_t n = getADCVal(adc);
	//if (abs((int16_t)(reg[r]-n))>1) {
	if (reg[r]!=n) {
		reg[r] = n;
		regUpdate[r] = 1;
	}
}

void updateButton(uint8_t r, uint8_t pos, uint8_t pin) {
	uint8_t button = (pin == 0);
	uint8_t reg_mask = 1<<pos;
	if (((reg[r] & reg_mask)>0) != button) {
		if (button)
			reg[r] |= reg_mask;
		else
			reg[r] &= ~reg_mask;
		regUpdate[r] = 1;
	}
}

int main (void) {
#ifdef USELCD
	char s[32];
#endif

	// disable jtag
	MCUCSR = (1<<JTD);
	MCUCSR = (1<<JTD);
	
	adc_init();
	timer0_init();
#ifdef USELCD
    lcd_init(LCD_DISP_ON);
#endif
	encode_init();

	sei();
	initSpi();
        
	while(1) {

		updateEncoder(REG_ENCODER0, encodeRead());
		updateEncoder(REG_ENCODER1, encodeRead1());
		updateEncoder(REG_ENCODER2, encodeRead2());

		updateADC(REG_ADC0,0);

		updateButton(REG_BUTTON0, 0, BUTTON1_PIN); 
		updateButton(REG_BUTTON0, 1, BUTTON2_PIN); 

		spiUpdate();

		if (reg[REG_UPDATE] & 1) {
			regUpdate[REG_ENCODER0] = 1;
			regUpdate[REG_ENCODER1] = 1;
			regUpdate[REG_ENCODER2] = 1;
			regUpdate[REG_ADC0] = 1;
			reg[REG_UPDATE] &= ~1;
		}

#ifdef USELCD 
		lcd_clrscr();
		sprintf(s, "%i",getADCVal(0));
		//sprintf(s, "%i",reg[REG_ENCODER0]);
		lcd_puts(s);
		lcd_gotoxy(0,1);
		sprintf(s, "%i",reg[REG_ENCODER1]);
		lcd_puts(s);
		_delay_ms(200);
#else
		_delay_ms(10);
#endif

	};
	return (0);
}
